package com.xyhy.jiaranmqttcluster.common.mqtt;

import com.xyhy.jiaranmqttcluster.common.Common;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;


public class MqttServer extends Common {


    private static MqttServer _mqttServer;

    private MqttClient _client;

    private MqttGetMessage pushCallback;

    private String host = Common.getConfigByName("MQTT.server.hostUrl");

    private String clientId = Common.getConfigByName("project.name");

    private String username = Common.getConfigByName("MQTT.server.hostName");

    private String password = Common.getConfigByName("MQTT.server.password");

    private int timeout = Integer.parseInt(Common.getConfigByName("MQTT.server.timeout"));

    private int keepAlive = Integer.parseInt(Common.getConfigByName("MQTT.server.keepAlive"));

    //@Value("#{'${MQTT.server.Subscribe}'.split(',')}")
    private String[] subscribe;

    public static MqttServer getInstance(){
        if (_mqttServer == null){
            _mqttServer = new MqttServer();
        }
        return _mqttServer;
    }

    public void connect(){
        _mqttServer.connect(host, clientId, username, password, timeout,keepAlive);
    }


    /**
     * 客户端连接
     *
     * @param host      ip+端口
     * @param clientID  客户端Id
     * @param username  用户名
     * @param password  密码
     * @param timeout   超时时间
     * @param keeplive 保留数
     */
    public void connect(String host,String clientID,String username,String password,int timeout,int keeplive){
        log.info("启动连接");
        MqttClient client;
        try {
            client=new MqttClient(host,clientID,new MemoryPersistence());
            MqttConnectOptions options=new MqttConnectOptions();
            options.setCleanSession(true);
            options.setUserName(username);
            options.setPassword(password.toCharArray());
            options.setConnectionTimeout(timeout);
            options.setKeepAliveInterval(keeplive);
            _client = client;
            try {
                client.setCallback(new MqttGetMessage());
                client.connect(options);
                log.info("MQTT 服务连接成功!");
                configSubscribe();
            }catch (Exception e){
                e.printStackTrace();
                log.error("MQTT 服务连接失败！",e);
            }
        }catch (Exception e){
            e.printStackTrace();
            log.error("MQTT 服务连接失败！",e);
        }
    }

    void configSubscribe(){

        subscribe = Common.getConfigByName("MQTT.server.Subscribe").split(",");
        for (String topic : subscribe){
            addSub(topic);
        }
    }

    public void addSub(String topic) {
        try {
            _client.subscribe(topic,1);
            log.info("MQTT 添加订阅主题成功！TOPIC："+topic);
        } catch (MqttException e) {
            e.printStackTrace();
            log.error("MQTT 添加订阅主题失败！TOPIC："+topic,e);
        }
    }

    /**
     * 发布
     * @param topic       主题
     * @param pushMessage 消息体
     */
    public void pushlish(String topic,String pushMessage){
        log.info("发送消息："+ pushMessage);
        MqttMessage message=new MqttMessage();
        message.setQos(1);
        message.setRetained(false);
        message.setPayload(pushMessage.getBytes());
        MqttTopic mqttTopic= _client.getTopic(topic);
        if(null== mqttTopic){
            log.error("topic not exist");
        }
        MqttDeliveryToken token;
        try {
            token=mqttTopic.publish(message);
            token.waitForCompletion();
        }catch (MqttPersistenceException e){
            e.printStackTrace();
        }catch (MqttException e){
            e.printStackTrace();
        }
    }

    public void reConnect(){
        try {
            _client.reconnect();
        }catch (MqttException e) {
            e.printStackTrace();
        }
    }
}
